#! python3  # noqa: E265

"""
    Tools to manage the environment setup (variables, etc.)

    Author: Julien Moura (https://github.com/guts)
"""

# #############################################################################
# ########## Libraries #############
# ##################################


# Standard library
import logging
import os
from os.path import expanduser, expandvars
from pathlib import Path
from sys import platform as opersys

# package
from qgis_deployment_toolbelt.jobs.generic_job import GenericJob
from qgis_deployment_toolbelt.utils.check_path import (
    check_path_exists,
    check_var_can_be_path,
)
from qgis_deployment_toolbelt.utils.url_helpers import check_str_is_url

# conditional import
if opersys == "linux":
    from qgis_deployment_toolbelt.utils.linux_utils import (
        delete_environment_variable,
        refresh_environment,
        set_environment_variable,
    )
elif opersys == "win32":
    from qgis_deployment_toolbelt.utils.win32utils import (
        delete_environment_variable,
        refresh_environment,
        set_environment_variable,
    )

# #############################################################################
# ########## Globals ###############
# ##################################


# logs
logger = logging.getLogger(__name__)

# #############################################################################
# ########## Classes ###############
# ##################################


class JobEnvironmentVariables(GenericJob):
    """
    Class to manage the environment variables of QGIS installation.
    """

    ID: str = "manage-env-vars"
    OPTIONS_SCHEMA: dict = {
        "action": {
            "type": str,
            "required": False,
            "default": "add",
            "possible_values": ("add", "remove"),
            "condition": "in",
        },
        "name": {
            "type": str,
            "required": True,
            "default": None,
            "possible_values": None,
            "condition": None,
        },
        "scope": {
            "type": str,
            "required": False,
            "default": "user",
            "possible_values": ("system", "user"),
            "condition": "in",
        },
        "value": {
            "type": (bool, int, str, list),
            "required": False,
            "default": None,
            "possible_values": None,
            "condition": None,
        },
        "value_type": {
            "type": str,
            "required": False,
            "default": "str",
            "possible_values": ("bool", "path", "str", "url"),
            "condition": "in",
        },
    }

    def __init__(self, options: list[dict]) -> None:
        """Instantiate the class.
        Args:
            options (List[dict]): list of dictionary with environment variables to set
            or remove.
        """

        super().__init__()
        self.options: list[dict] = [self.validate_options(opt) for opt in options]

    def run(self) -> None:
        """Apply environment variables from dictionary to the system."""

        if opersys not in ("linux", "win32"):
            logger.error(f"This job does not support your operating system: {opersys}")
            return

        for env_var in self.options:
            if env_var.get("action") == "add":
                try:
                    envvar_name = env_var.get("name", None)
                    envvar_value = self.prepare_value(
                        value=env_var.get("value", None),
                        value_type=env_var.get("value_type", None),
                    )
                    set_environment_variable(
                        envvar_name=envvar_name,
                        envvar_value=envvar_value,
                        scope=env_var.get("scope", None),
                    )
                    # Add to local environnement to be used in other QDT jobs
                    os.environ[envvar_name] = envvar_value
                except NameError:
                    logger.debug(
                        f"Variable name '{env_var.get('name')}' is not defined"
                    )
            elif env_var.get("action") == "remove":
                try:
                    envvar_name = env_var.get("name", None)
                    delete_environment_variable(
                        envvar_name=envvar_name,
                        scope=env_var.get("scope", None),
                    )
                    # Remove from local environnement so it won't be used in other QDT jobs
                    if envvar_name in os.environ:
                        del os.environ[envvar_name]
                except NameError:
                    logger.debug(
                        f"Variable name '{env_var.get('name')}' is not defined"
                    )

        # refresh environment to update variables
        refresh_environment()

        # log
        logger.debug(f"Job {self.ID} ran successfully.")

    # -- INTERNAL LOGIC ------------------------------------------------------
    def prepare_value(self, value: str, value_type: str | None = None) -> str:
        """Prepare value to be used in the environment variable.

        It performs some checks or operations depending on value type: user and
            variable expansion, check if URL is valid, etc.

        Args:
            value (str): value to prepare.
            value_type (str, optional): type of input value. Defaults to "str".

        Returns:
            str: prepared value.
        """

        if value_type == "url":
            if check_str_is_url(input_str=value, raise_error=False):
                logger.info(
                    f"{value} is a valid URL. Using it as environment variable value."
                )
            else:
                logger.warning(
                    f"{value} seems to be an invalid URL. " "It will be applied anyway."
                )

            return value.strip()
        elif value_type in ("bool", "str"):
            return str(value).strip()
        elif value_type == "path":
            # test if value is a path
            if check_var_can_be_path(input_var=value, raise_error=False):
                value_as_path = Path(expanduser(expandvars(value)))
                if not check_path_exists(input_path=value_as_path, raise_error=False):
                    logger.warning(
                        f"{value} seems to be a valid path but does not exist (yet)."
                    )
                return str(Path(value_as_path).resolve())
            else:
                logger.debug(
                    f"Value {value} is not a valid path. The raw string will be used."
                )

        return str(value).strip()



#! python3  # noqa E265

"""
    Usage from the repo root folder:

    .. code-block:: bash
        # for whole tests
        python -m unittest tests.test_utils
        # for specific test
        python -m unittest tests.test_utils.TestUtilsWin32.test_win32_getenv
"""

# standard library
import unittest
from sys import platform as opersys

# project
from qgis_deployment_toolbelt.utils.win32utils import get_environment_variable

# ############################################################################
# ########## Classes #############
# ################################


class TestUtilsWin32(unittest.TestCase):
    """Test package utilities."""

    @unittest.skipIf(opersys != "win32", "Test specific to Windows.")
    def test_win32_getenv(self):
        """Test specific Windows environment variable getter."""
        # OK
        self.assertIsInstance(get_environment_variable("TEMP"), str)

        # KO
        self.assertIsNone(get_environment_variable("YOUPI"))


# ############################################################################
# ####### Stand-alone run ########
# ################################
if __name__ == "__main__":
    unittest.main()



#! python3  # noqa: E265

"""
    Handle remote HTTP repository.

    Author: Julien Moura (https://github.com/guts).
"""


# #############################################################################
# ########## Libraries #############
# ##################################


# Standard library
import logging
from concurrent.futures import ThreadPoolExecutor
from os import getenv
from pathlib import Path
from shutil import rmtree

# 3rd party
import requests

# project
from qgis_deployment_toolbelt.__about__ import __title_clean__, __version__
from qgis_deployment_toolbelt.profiles.profiles_handler_base import (
    RemoteProfilesHandlerBase,
)
from qgis_deployment_toolbelt.utils.file_downloader import download_remote_file_to_local
from qgis_deployment_toolbelt.utils.formatters import url_ensure_trailing_slash
from qgis_deployment_toolbelt.utils.proxies import get_proxy_settings
from qgis_deployment_toolbelt.utils.str2bool import str2bool
from qgis_deployment_toolbelt.utils.tree_files_reader import tree_to_download_list

# #############################################################################
# ########## Globals ###############
# ##################################

# logs
logger = logging.getLogger(__name__)

# #############################################################################
# ########## Classes ###############
# ##################################


class HttpHandler(RemoteProfilesHandlerBase):
    """Handle remote HTTP repositories without git protocol.

    It's designed to handle thoses cases:

    - the distant repository (source) is on a webserver accessible through HTTP protocol
    - the local repository (destination) is on a local network or drive
    """

    # headers
    HTTP_HEADERS = {
        "User-Agent": f"{__title_clean__}/{__version__}",
    }

    def __init__(
        self,
        source_repository_path_or_uri: str,
        source_repository_type: str = "http",
    ) -> None:
        """Constructor.

        Args:
            source_repository_path_or_uri (str | Path): path to the source repository
        """
        self.SOURCE_REPOSITORY_PATH_OR_URL = url_ensure_trailing_slash(
            source_repository_path_or_uri
        )
        super().__init__(source_repository_type=source_repository_type)

    def download(self, destination_local_path: Path):
        """Generic wrapper around the specific logic of this handler.

        Args:
            destination_local_path (Path): path to the local folder where to download

        """
        logger.info(
            f"Start downloading from {self.SOURCE_REPOSITORY_PATH_OR_URL} to "
            f"{destination_local_path}"
        )

        try:
            logger.debug("Retrieve qdt-files.json ")
            # get qdt-files.json
            req = requests.get(
                url=f"{self.SOURCE_REPOSITORY_PATH_OR_URL}qdt-files.json",
                headers=self.HTTP_HEADERS,
                proxies=get_proxy_settings(
                    url=f"{self.SOURCE_REPOSITORY_PATH_OR_URL}qdt-files.json"
                ),
            )
            req.raise_for_status()
            qdt_tree = req.json()
        except Exception as err:
            logger.critical(
                f"Downloading {self.SOURCE_REPOSITORY_PATH_OR_URL} to "
                f"{destination_local_path} failed. Trace: {err}"
            )
            raise err

        # clean everything before downloading
        rmtree(path=destination_local_path, ignore_errors=True)

        # make sure destination path exists
        if not destination_local_path.exists():
            logger.debug(
                f"Destination folder ({destination_local_path}) does not exist. "
                "Let's create it!"
            )
            destination_local_path.mkdir(parents=True)

        li_files_to_download = tree_to_download_list(tree_array=qdt_tree)
        logger.info(f"{len(li_files_to_download)} files to download")

        success, fails = self.download_files_to_local(
            li_files_to_download=li_files_to_download,
            target_folder=destination_local_path,
        )
        if not len(success):
            logger.error("No files downloaded! Please check the above log messages.")
        if len(fails):
            logger.warning(
                f"{len(fails)} download failed. Check the above log messages."
            )

    def download_files_to_local(
        self, li_files_to_download: list[str], target_folder: Path
    ) -> tuple[list[tuple[str, Path]], list[tuple[str, str]]]:
        """Download list of files relative to remote base URL to local target folder.

        Args:
            li_files_to_download (list[str]): list of files to download.
            target_folder (Path): local folder where to download

        Returns:
            (list of success download, list of failed download)
        """
        base_url = self.SOURCE_REPOSITORY_PATH_OR_URL
        downloaded_files: list[tuple[str, Path]] = []
        failed_files: list[tuple[str, str]] = []

        with ThreadPoolExecutor(
            thread_name_prefix=f"{__title_clean__}_profile_dl_http_"
        ) as executor:
            for file_to_download in li_files_to_download:
                # submit download to pool
                try:
                    executor.submit(
                        # func to execute
                        download_remote_file_to_local,
                        # func parameters
                        local_file_path=target_folder.joinpath(file_to_download),
                        remote_url_to_download=f"{base_url}{file_to_download}",
                        use_stream=str2bool(getenv("QDT_STREAMED_DOWNLOADS", True)),
                    )
                    downloaded_files.append(
                        (
                            f"{base_url}{file_to_download}",
                            target_folder.joinpath(file_to_download),
                        )
                    )
                except Exception as err:
                    failed_files.append((file_to_download, f"{err}"))

        return downloaded_files, failed_files



#! python3  # noqa: E265

"""
    Basic string to bool.
"""

# #############################################################################
# ########## Libraries #############
# ##################################

# Standard library
import logging

# #############################################################################
# ########## Globals ###############
# ##################################

_true_set = {"on", "t", "true", "y", "yes", "1"}
_false_set = {"f", "false", "n", "no", "off", "0"}
logger = logging.getLogger(__name__)

# #############################################################################
# ########## Functions #############
# ##################################


def str2bool(input_var: bool | str, raise_exc: bool = False) -> bool | None:
    """Determine if a string is a bool and, if so, convert it.

    Args:
        input_var (str): value to convert
        raise_exc (bool, optional): strict mode, defaults to False. Defaults to False.

    Raises:
        ValueError: input_var is not in true and false sets
        TypeError: if input_var is not a str or a bool

    Returns:
        bool: True if input_var is in _true_set, False if it's in _false_set.
    None or Exception if not in any of two sets.
    """
    if isinstance(input_var, str):
        value = input_var.lower()
        if value in _true_set:
            return True
        if value in _false_set:
            return False

        error_message = 'Expected "%s"' % '", "'.join(_true_set | _false_set)
        if raise_exc:
            raise ValueError(error_message)
        else:
            logger.error(error_message)
            return None
    elif isinstance(input_var, bool):
        logger.debug(f"Value {input_var} was already a bool.")
        return input_var
    else:
        error_message = f"Value must be a str, not {type(input_var)}"
        if raise_exc:
            raise TypeError(error_message)
        else:
            logger.error(error_message)
            return None



#! python3  # noqa E265

"""
    Usage from the repo root folder:

    .. code-block:: bash
        # for whole tests
        python -m unittest tests.test_utils_bouncer
        # for specific test
        python -m unittest tests.test_utils_bouncer.TestUtilsBouncer.test_bouncer_error
"""

# standard library
import unittest

# project
from qgis_deployment_toolbelt.utils.bouncer import (
    exit_cli_error,
    exit_cli_normal,
    exit_cli_success,
)

# ############################################################################
# ########## Classes #############
# ################################


class TestUtilsBouncer(unittest.TestCase):
    """Test package utilities."""

    def test_bouncer_error(self):
        """Test bouncer error."""
        # test with simple string
        error_message = "fake error"

        with self.assertRaises(SystemExit) as err:
            exit_cli_error(error_message)
        exception = err.exception
        self.assertEqual(exception.code, error_message)

        self.assertIsNone(exit_cli_error(error_message, abort=False))

        # test with an Exception
        error = TypeError("fake type error")
        with self.assertRaises(SystemExit):
            exit_cli_error(error)
        self.assertIsNone(exit_cli_error(error, abort=False))

    def test_bouncer_normal(self):
        """Test bouncer normal."""
        # test with simple string
        error_message = "fake error"

        with self.assertRaises(SystemExit) as err:
            exit_cli_normal(error_message)
        exception = err.exception
        self.assertEqual(exception.code, 0)

        self.assertIsNone(exit_cli_normal(error_message, abort=False))

    def test_bouncer_success(self):
        """Test bouncer success."""
        # test with simple string
        error_message = "fake error"

        with self.assertRaises(SystemExit) as err:
            exit_cli_success(error_message)
        exception = err.exception
        self.assertEqual(exception.code, 0)

        self.assertIsNone(exit_cli_success(error_message, abort=False))


# ############################################################################
# ####### Stand-alone run ########
# ################################
if __name__ == "__main__":
    unittest.main()



#! python3  # noqa: E265

"""
    Helpers to check file: readable, exists, etc..

    Author: Julien Moura (https://github.com/guts)
"""

# #############################################################################
# ########## Libraries #############
# ##################################

# Standard library
import logging
from os import R_OK, W_OK, access
from os.path import expanduser, expandvars
from pathlib import Path

# #############################################################################
# ########## Globals ###############
# ##################################

# logs
logger = logging.getLogger(__name__)

# #############################################################################
# ########## Functions #############
# ##################################


def check_folder_is_empty(input_var: Path, raise_error: bool = True) -> bool:
    """Check if a variable is valid path, exists, is a folder and is empty.

    Args:
        input_var (Path): path to check
        raise_error (bool, optional): if True, it raises an exception. Defaults to True.

    Returns:
        bool: True if the path is pointing to an empty folder
    """
    return check_path(
        input_path=input_var,
        must_be_a_file=False,
        must_be_a_folder=True,
        must_be_readable=True,
        must_exists=True,
        raise_error=raise_error,
    ) and not any(input_var.iterdir())


def check_var_can_be_path(
    input_var: str, attempt: int = 1, raise_error: bool = True
) -> bool:
    """Check is the path can be converted as pathlib.Path.

    Args:
        input_var (str): var to check
        attempt (int): attempt count. If attempts == 2, it tries expanding user and variables.
        raise_error (bool, optional): if True, it raises an exception. Defaults to True.

    Raises:
        TypeError: if input path can't be converted and raise_error is False
        ValueError: if input path is an empty string

    Returns:
        bool: True if the input can be converted to pathlib.Path
    """
    if isinstance(input_var, str) and not len(input_var.strip()):
        error_message = "Input var must have at least one character."
        if raise_error:
            raise ValueError(error_message)
        else:
            logger.error(error_message)
            return False

    try:
        if attempt == 2:
            input_var = Path(expandvars(expanduser(input_var)))
        else:
            input_var = Path(input_var)
        return True
    except Exception as exc:
        error_message = f"Converting {input_var} into Path failed. Trace: {exc}"
        if attempt != 2:
            error_message += " Attempt 1/2. Try again with user and vars expansion."
            logger.info(error_message)
            return check_var_can_be_path(
                input_var=input_var, attempt=2, raise_error=raise_error
            )

        error_message += " Attempt 2/2. Game over."
        if raise_error:
            raise TypeError(error_message)
        else:
            logger.error(error_message)
            return False


def check_path_exists(
    input_path: str | Path, attempt: int = 1, raise_error: bool = True
) -> bool:
    """Check if the input path (file or folder) exists.

    Args:
        input_path (Union[str, Path]): path to check
        attempt (int): attempt count. If attempts == 2, it tries expanding user and variables.
        raise_error (bool, optional): if True, it raises an exception. Defaults to True.

    Raises:
        FileExistsError: if the path doesn't exist and raise_error is False

    Returns:
        bool: True if the path exists.
    """
    if not isinstance(input_path, Path):
        if (
            not check_var_can_be_path(input_path, raise_error=raise_error)
            and not raise_error
        ):
            return False
        # if previous check passed, let's convert it safely
        input_path = Path(input_path)

    # if second attempt, try to expand user and vars

    if attempt == 2:
        input_path = Path(expandvars(expanduser(input_path)))
    else:
        input_path = Path(input_path)

    if not input_path.exists():
        error_message = f"{input_path.resolve()} doesn't exist."
        if attempt != 2:
            error_message += " Attempt 1/2. Try again with user and vars expansion."
            logger.info(error_message)
            return check_path_exists(
                input_path=input_path, attempt=2, raise_error=raise_error
            )

        error_message += " Attempt 2/2. Game over."

        if raise_error:
            raise FileExistsError(error_message)
        else:
            logger.error(error_message)
            return False
    else:
        return True


def check_path_is_readable(input_path: Path, raise_error: bool = True) -> bool:
    """Check if the input path (file or folder) is readable.

    Args:
        input_path (Path): path to check
        raise_error (bool, optional): if True, it raises an exception. Defaults to True.

    Raises:
        FileExistsError: if the path is not readable and raise_error is False

    Returns:
        bool: True if the path is readable.
    """
    # firstly check the path is valid and exists
    if not isinstance(input_path, Path):
        if (
            not check_path_exists(input_path, raise_error=raise_error)
            and not raise_error
        ):
            return False
        # if previous check passed, let's convert it safely
        input_path = Path(input_path)

    if not access(input_path, R_OK):
        error_message = f"{input_path.resolve()} isn't readable."
        if raise_error:
            raise OSError(error_message)
        else:
            logger.error(f"{input_path.resolve()} isn't readable.")
            return False
    else:
        return True


def check_path_is_writable(input_path: Path, raise_error: bool = True) -> bool:
    """Check if the input path (file or folder) is writable.

    Args:
        input_path (Path): path to check
        raise_error (bool, optional): if True, it raises an exception. Defaults to True.

    Raises:
        FileExistsError: if the path is not writable and raise_error is False

    Returns:
        bool: True if the path is writable.
    """
    # firstly check the path is valid and exists
    if not isinstance(input_path, Path):
        if (
            not check_path_exists(input_path, raise_error=raise_error)
            and not raise_error
        ):
            return False
        # if previous check passed, let's convert it safely
        input_path = Path(input_path)

    if not access(input_path, W_OK):
        error_message = f"{input_path.resolve()} isn't writable."
        if raise_error:
            raise OSError(error_message)
        else:
            logger.error(error_message)
            return False
    else:
        return True


def check_path(
    input_path: str | Path,
    must_exists: bool = True,
    must_be_readable: bool = True,
    must_be_writable: bool = False,
    must_be_a_folder: bool = False,
    must_be_a_file: bool = False,
    raise_error: bool = True,
) -> bool:
    """Meta function of the module. Check if a given path complies with some constraints.

    Args:
        input_path (Union[str, Path]): path to check
        must_exists (bool, optional): path must exist. Defaults to True.
        must_be_readable (bool, optional): path must be readable. Defaults to True.
        must_be_writable (bool, optional): path must be writable. Defaults to False.
        must_be_a_folder (bool, optional): path must be a folder. Mutually exclusive \
            with must_be_a_file. Defaults to False.
        must_be_a_file (bool, optional): path must be a file. Mutually exclusive with \
            must_be_a_folder. Defaults to False.
        raise_error (bool, optional): if True, it raises an exception. Defaults to True.

    Raises:
        ValueError: if must_be_a_file and must_be_a_folder are both set to True
        FileNotFoundError: if the path is not a file and must_be_a_file is set to True
        NotADirectoryError: if the path is not a folder and must_be_a_folder is set to True

    Returns:
        bool:  True if the path complies with constraints.
    """
    # check arguments
    if all([must_be_a_file, must_be_a_folder]):
        raise ValueError(
            "These options are mutually exclusive: must_be_a_file, must_be_a_folder"
        )

    # check input path if usable with pathlib.Path
    if not isinstance(input_path, Path):
        check_var = check_var_can_be_path(input_var=input_path, raise_error=raise_error)
        if not check_var and not raise_error:
            return False
        input_path = Path(input_path)

    # check
    if must_exists:
        check_exist = check_path_exists(input_path=input_path, raise_error=raise_error)
        if not check_exist and not raise_error:
            return False

    # check file or folder
    if must_be_a_file and not input_path.is_file():
        error_message = f"{input_path.resolve()} is not a file."
        if raise_error:
            raise FileNotFoundError(error_message)
        else:
            logger.error(error_message)
            return False
    if must_be_a_folder and not input_path.is_dir():
        error_message = f"{input_path.resolve()} is not a folder."
        if raise_error:
            raise NotADirectoryError(error_message)
        else:
            logger.error(error_message)
            return False

    # check chmod
    if must_be_readable:
        check_readable = check_path_is_readable(
            input_path=input_path, raise_error=raise_error
        )
        if not check_readable and not raise_error:
            return False

    if must_be_writable:
        check_writable = check_path_is_writable(
            input_path=input_path, raise_error=raise_error
        )
        if not check_writable and not raise_error:
            return False

    return True


# ############################################################################
# ##### Stand alone program ########
# ##################################

if __name__ == "__main__":
    """Standalone execution."""
    check_var_can_be_path(input_var="~")
    check_path_exists(input_path="~")



#! python3  # noqa E265

"""Usage from the repo root folder:

    .. code-block:: python

        # for whole test
        python -m unittest tests.test_git_handler_local
        # for specific
        python -m unittest tests.test_git_handler_local.TestGitHandlerLocal.test_git_url_parsed
"""

# #############################################################################
# ########## Libraries #############
# ##################################

# Standard library
import tempfile
import unittest
from pathlib import Path
from shutil import rmtree

# 3rd party
from dulwich.errors import NotGitRepository
from dulwich.repo import Repo
from git import Repo as GitPythonRepo

# package
from qgis_deployment_toolbelt.profiles.local_git_handler import LocalGitHandler

# #############################################################################
# ########## Classes ###############
# ##################################


class TestGitHandlerLocal(unittest.TestCase):
    """Test module."""

    # -- Standard methods --------------------------------------------------------
    @classmethod
    def setUpClass(cls):
        """Executed when module is loaded before any test."""
        cls.remote_repo_url = "https://github.com/geotribu/profils-qgis.git"
        cls.source_git_path_source = Path("tests/fixtures/tmp/git_handler_local_source")
        cls.local_git_path_target = Path("tests/fixtures/tmp/git_handler_local_target")
        cls.source_local_repository_obj = GitPythonRepo.clone_from(
            url=cls.remote_repo_url, to_path=cls.source_git_path_source
        )

    @classmethod
    def tearDownClass(cls) -> None:
        """Executed before module is shutdown after every test."""
        cls.source_local_repository_obj.git.clear_cache()
        cls.source_local_repository_obj.git = None
        rmtree(path=cls.source_git_path_source, ignore_errors=True)
        rmtree(path=cls.local_git_path_target, ignore_errors=True)

    # -- TESTS ---------------------------------------------------------
    def test_initialization(self):
        """Test module instanciation."""
        # OK - with pathlib.Path
        local_git_handler = LocalGitHandler(
            source_repository_path_or_uri=self.source_git_path_source,
            branch_to_use="main",
        )

        self.assertIsInstance(local_git_handler.SOURCE_REPOSITORY_PATH_OR_URL, Path)
        self.assertEqual(local_git_handler.SOURCE_REPOSITORY_TYPE, "git_local")
        self.assertEqual(
            local_git_handler.SOURCE_REPOSITORY_PATH_OR_URL,
            self.source_git_path_source.resolve(),
        )
        self.assertEqual(local_git_handler.SOURCE_REPOSITORY_ACTIVE_BRANCH, "main")

        # OK - with str
        local_git_handler = LocalGitHandler(
            source_repository_path_or_uri=f"{self.source_git_path_source.resolve()}",
            branch_to_use="main",
        )
        self.assertEqual(
            local_git_handler.SOURCE_REPOSITORY_PATH_OR_URL,
            self.source_git_path_source.resolve(),
        )
        self.assertEqual(local_git_handler.SOURCE_REPOSITORY_ACTIVE_BRANCH, "main")

        # OK - with str but no branch
        local_git_handler = LocalGitHandler(
            source_repository_path_or_uri=f"{self.source_git_path_source.resolve()}",
        )
        self.assertEqual(
            local_git_handler.SOURCE_REPOSITORY_PATH_OR_URL,
            self.source_git_path_source.resolve(),
        )
        self.assertEqual(local_git_handler.SOURCE_REPOSITORY_ACTIVE_BRANCH, "main")
        self.assertIsInstance(local_git_handler.SOURCE_REPOSITORY_PATH_OR_URL, Path)

        # KO
        with tempfile.TemporaryDirectory(
            prefix="qdt_test_local_git_", ignore_cleanup_errors=True
        ) as tmpdirname:
            with self.assertRaises(NotGitRepository):
                LocalGitHandler(source_repository_path_or_uri=tmpdirname)

    def test_clone_with_specified_branch_existing(self):
        """Test clone with specified branch."""
        local_git_handler = LocalGitHandler(
            source_repository_path_or_uri=self.source_git_path_source,
            branch_to_use="main",
        )

        self.assertEqual(local_git_handler.SOURCE_REPOSITORY_TYPE, "git_local")

        with tempfile.TemporaryDirectory(
            prefix="QDT_test_local_git_",
            ignore_cleanup_errors=True,
            suffix="_specified_branch_existing",
        ) as tmpdirname:
            target_repo = local_git_handler.download(
                destination_local_path=Path(tmpdirname)
            )
            self.assertIsInstance(target_repo, Repo)

    def test_clone_with_specified_branch_not_existing(self):
        """Test clone with specified branch."""
        local_git_handler = LocalGitHandler(
            source_repository_path_or_uri=self.source_git_path_source,
            branch_to_use="no_existing_branch",
        )

        self.assertEqual(local_git_handler.SOURCE_REPOSITORY_TYPE, "git_local")
        self.assertEqual(local_git_handler.DESTINATION_BRANCH_TO_USE, "main")

        local_git_handler.download(
            destination_local_path=self.local_git_path_target.resolve()
        )

        with tempfile.TemporaryDirectory(
            prefix="QDT_test_local_git_",
            ignore_cleanup_errors=True,
            suffix="_specified_branch_not_existing",
        ) as tmpdirname:
            target_repo = local_git_handler.download(
                destination_local_path=Path(tmpdirname)
            )
            self.assertIsInstance(target_repo, Repo)

    def test_clone_then_fetch_pull(self):
        """Test clone with specified branch."""
        local_git_handler = LocalGitHandler(
            source_repository_path_or_uri=self.source_git_path_source,
            branch_to_use="main",
        )

        self.assertEqual(local_git_handler.SOURCE_REPOSITORY_TYPE, "git_local")

        with tempfile.TemporaryDirectory(
            prefix="QDT_test_local_git_",
            ignore_cleanup_errors=True,
            suffix="_clone_the_pull",
        ) as tmpdirname:
            repo_in_temporary_folder = Path(tmpdirname)

            target_repo = local_git_handler.download(
                destination_local_path=repo_in_temporary_folder
            )
            self.assertIsInstance(target_repo, Repo)
            self.assertTrue(repo_in_temporary_folder.joinpath(".git").is_dir())

            target_repo = local_git_handler.download(
                destination_local_path=repo_in_temporary_folder
            )



#! python3  # noqa E265

"""Usage from the repo root folder:

    .. code-block:: python

        # for whole test
        python -m unittest tests.test_job_profiles_sync
        # for specific
        python -m unittest tests.test_job_profiles_sync.TestJobProfilesSync.test_fresh_install
"""

# #############################################################################
# ########## Libraries #############
# ##################################

# Standard library
import unittest
from pathlib import Path

# package

# #############################################################################
# ########## Classes ###############
# ##################################


class TestJobProfilesSync(unittest.TestCase):
    """Test module."""

    # -- Standard methods --------------------------------------------------------
    @classmethod
    def setUpClass(cls):
        """Executed when module is loaded before any test."""
        cls.good_scenario_files = sorted(
            Path("tests/fixtures/").glob("scenarios/good_*.y*ml")
        )

    # standard methods
    def setUp(self):
        """Fixtures prepared before each test."""
        pass

    def tearDown(self):
        """Executed after each test."""
        pass

    # -- TESTS ---------------------------------------------------------
    # @unittest.skipIf(not getenv("CI", False), "Skip on local machine")
    # def test_fresh_install(self):
    #     """Test QGIS fresh new installation where no profile is installed yet."""
    #     # variables
    #     fake_config = {
    #         "action": "download",
    #         "source": "https://gitlab.com/Oslandia/qgis/profils_qgis_fr.git",
    #         "protocol": "git",
    #         "local_destination": "~/.cache/qgis-deployment-toolbelt/Oslandia/",
    #         "sync_mode": "overwrite",
    #     }
    #     qgis_profiles_path: Path = Path(OS_CONFIG.get(opersys).profiles_path)

    #     # QGIS profiles folder should not exist yet
    #     self.assertFalse(qgis_profiles_path.exists())

    #     # after instanciation, the folder should exist
    #     profile_manager = JobProfilesDownloader(options=fake_config)
    #     self.assertTrue(profile_manager.qgis_profiles_path.exists())



#! python3  # noqa E265

"""
    Usage from the repo root folder:

    .. code-block:: bash
        # for whole tests
        python -m unittest tests.test_user_groups
        # for specific test
        python -m unittest tests.test_user_groups.TestUtilsUserGroups.test_win32_getenv
"""

# standard library
import unittest
from getpass import getuser
from sys import platform as opersys

# project
from qgis_deployment_toolbelt.utils.user_groups import (
    _is_computer_in_domain_powershell,
    _is_computer_in_domain_pyad,
    _is_computer_in_domain_win32,
    get_user_local_groups,
    is_computer_attached_to_a_domain,
    is_user_in_group,
)

# ############################################################################
# ########## Classes #############
# ################################


class TestUtilsUserGroups(unittest.TestCase):
    """Test package utilities."""

    def test_computer_attached_to_domain(self):
        """Test if the computer belongs to a domain."""
        # for now, domain link is only supported on Windows
        if opersys != "win32":
            self.assertFalse(is_computer_attached_to_a_domain())
            return

        # test different methods on Windows but for now, no domain mocked in tests
        # TODO: mock domain link
        domain_status = is_computer_attached_to_a_domain()
        self.assertIsInstance(domain_status, bool)

        self.assertEqual(domain_status, _is_computer_in_domain_powershell())
        self.assertEqual(domain_status, _is_computer_in_domain_pyad())
        self.assertEqual(domain_status, _is_computer_in_domain_win32())

    def test_user_in_group(self):
        """Test function determining user's groups."""
        user_groups = get_user_local_groups()

        # test with existing group
        self.assertTrue(is_user_in_group(user_groups[0]))

        # test with unisting group
        self.assertFalse(is_user_in_group(group_name="fake_group", user_name=getuser()))


# ############################################################################
# ####### Stand-alone run ########
# ################################
if __name__ == "__main__":
    unittest.main()



#! python3  # noqa: E265

"""
    Shortcuts to exit CLI properly and with pretty messages.

    Author: Julien Moura (github.com/guts)
"""

# #############################################################################
# ########## Libraries #############
# ##################################

# Standard library
import logging
import sys

# project
from qgis_deployment_toolbelt.utils.journalizer import get_logger_filepath

# #############################################################################
# ########## Globals ###############
# ##################################

# logs
logger = logging.getLogger(__name__)

# #############################################################################
# ########## Functions #############
# ##################################


def exit_cli_error(message: str | Exception, abort: bool = True):
    """Display error message and stop execution.

    Args:
        message (Union[str, Exception]): message to log and display in terminal.
        abort (bool, optional): option to abort after displaying. Defaults to True.

    Raises:
        SystemExit: when abort is True
    """
    # log
    logger.error(message, exc_info=True)
    logger.error(f"Please, read the full detailed log: {get_logger_filepath()}")

    # handle cases when the full exception is passed
    if isinstance(message, Exception):
        if isinstance(message.args, tuple) and len(message.args):
            message = message.args[0]
        else:
            message = getattr(message, "message", repr(message))
    if abort:
        sys.exit(message)


def exit_cli_normal(message: str | Exception, abort: bool = True):
    """Display normal message and stop execution if required.

    Args:
        message (Union[str, Exception]): message to log and display in terminal.
        abort (bool, optional): option to abort after displaying. Defaults to True.

    Raises:
        SystemExit: when abort is True
    """
    logger.info(message)

    if abort:
        sys.exit(0)


def exit_cli_success(message: str | Exception, abort: bool = True):
    """Display success message and stop execution ir required.

    Args:
        message (Union[str, Exception]): message to log and display in terminal.
        abort (bool, optional): option to abort after displaying. Defaults to True.

    Raises:
        SystemExit: when abort is True
    """
    logger.info(message)

    if abort:
        sys.exit(0)


# #############################################################################
# ##### Stand alone program ########
# ##################################

if __name__ == "__main__":
    """Standalone execution."""
    pass



#! python3  # noqa: E265

"""
    Small module to get network proxies configuration.

    Author: Julien Moura (github.com/guts)
"""

# #############################################################################
# ########## Libraries #############
# ##################################

# Standard library
import logging
import os
from functools import lru_cache
from os import environ
from urllib.request import getproxies

# 3rd party
from pypac import get_pac, pac_context_for_url
from pypac.parser import PACFile

# package
from qgis_deployment_toolbelt.utils.url_helpers import check_str_is_url

# #############################################################################
# ########## Globals ###############
# ##################################

# logs
logger = logging.getLogger(__name__)


# #############################################################################
# ########## Functions #############
# ##################################
@lru_cache
def get_proxy_settings(url: str | None = None) -> dict:
    """Retrieves network proxy settings from operating system configuration or
    environment variables.
    Args:
        url (str, optional): url for request in case of PAC file use
    Returns:
        dict: proxy settings with protocl as key and URL as value
    """

    proxy_settings = {}
    if environ.get("QDT_PROXY_HTTP"):
        proxy_settings = {
            "http": environ.get("QDT_PROXY_HTTP"),
            "https": environ.get("QDT_PROXY_HTTP"),
        }
        logger.info(
            "Proxies settings from custom QDT in environment vars (QDT_PROXY_HTTP): "
            f"{proxy_settings}"
        )
    elif qdt_pac_file := environ.get("QDT_PAC_FILE"):
        if pac := load_pac_file_from_environment_variable(qdt_pac_file=qdt_pac_file):
            proxy_settings = get_proxy_settings_from_pac_file(url=url, pac=pac)
            logger.info(
                f"Proxies settings from environment vars PAC file: {environ.get('QDT_PAC_FILE')}"
                f"{proxy_settings}"
            )
        else:
            logger.warning(
                f"Invalid PAC file from environment vars PAC file : {environ.get('QDT_PAC_FILE')}. No proxy use."
            )
    elif pac := get_pac():
        proxy_settings = get_proxy_settings_from_pac_file(url=url, pac=pac)
        logger.info("Proxies settings from system PAC file: " f"{proxy_settings}")
    elif getproxies():
        proxy_settings = getproxies()
        logger.debug(f"Proxies settings found in the OS: {proxy_settings}")
    elif environ.get("HTTP_PROXY") or environ.get("HTTPS_PROXY"):
        if environ.get("HTTP_PROXY") and environ.get("HTTPS_PROXY"):
            proxy_settings = {
                "http": environ.get("HTTP_PROXY"),
                "https": environ.get("HTTPS_PROXY"),
            }
            logger.info(
                "Proxies settings from generic environment vars (HTTP_PROXY "
                f"and HTTPS_PROXY): {proxy_settings}"
            )
        elif environ.get("HTTP_PROXY") and not environ.get("HTTPS_PROXY"):
            proxy_settings = {
                "http": environ.get("HTTP_PROXY"),
            }
            logger.info(
                "Proxies settings from generic environment vars (HTTP_PROXY only): "
                f"{proxy_settings}"
            )
        elif not environ.get("HTTP_PROXY") and environ.get("HTTPS_PROXY"):
            proxy_settings = {
                "https": environ.get("HTTPS_PROXY"),
            }
            logger.info(
                "Proxies settings from generic environment vars (HTTPS_PROXY only): "
                f"{proxy_settings}"
            )
    else:
        logger.debug(
            "No proxy settings found in environment vars nor OS settings nor PAC File."
        )

    # check scheme and URL validity
    if isinstance(proxy_settings, dict):
        for scheme, proxy_url in proxy_settings.items():
            if not check_str_is_url(input_str=proxy_url, raise_error=False):
                logger.warning(
                    f"Proxy value for {scheme} is not a valid URL: {proxy_url}. Can "
                    "lead to troubles."
                )

    return proxy_settings


def load_pac_file_from_environment_variable(qdt_pac_file: str) -> PACFile | None:
    """Load PAC file with PyPAC from a environment variable

    Args:
        qdt_pac_file (str): path to PAC file

    Returns:
        Optional[PACFile]: loaded PAC file, None if value is invalid
    """
    if qdt_pac_file.startswith(("http",)):
        return get_pac(
            qdt_pac_file,
            allowed_content_types=[
                "text/plain",
                "application/x-ns-proxy-autoconfig",
                "application/x-javascript-config",
            ],
        )
    else:
        with open(qdt_pac_file, encoding="UTF-8") as f:
            return PACFile(f.read())


def get_proxy_settings_from_pac_file(
    pac: PACFile, url: str | None = None
) -> dict[str, str]:
    """Define proxy settings from pac file

    Args:
        url (str): url for request in case of PAC file use
        pac (PACFile): _description_

    Returns:
        dict[str, str]: _description_
    """

    proxy_settings = {}
    with pac_context_for_url(url=url, pac=pac):
        if environ.get("HTTP_PROXY"):
            proxy_settings["http"] = environ.get("HTTP_PROXY")
        if environ.get("HTTPS_PROXY"):
            proxy_settings["https"] = environ.get("HTTPS_PROXY")
    return proxy_settings


def os_env_proxy(func):
    def wrapper(*args, **kwargs):
        """Decorator wrapper to define environment variable for proxy use.

        If a proxy settings is available for https or http we:
        - backup current environment value
        - define environment value with proxy settings
        - run function
        - restore environment value if available


        Returns:
            _type_: function result
        """
        # Get proxy settings
        proxy_settings = get_proxy_settings()

        # Update environment variable and keep current value
        prev_http_proxy = None
        if "http" in proxy_settings:
            prev_http_proxy = environ.get("HTTP_PROXY")
            os.environ["HTTP_PROXY"] = proxy_settings["http"]
        prev_https_proxy = None
        if "https" in proxy_settings:
            prev_https_proxy = environ.get("HTTPS_PROXY")
            os.environ["HTTPS_PROXY"] = proxy_settings["https"]

        # Run function
        result = func(*args, **kwargs)

        # Restore environment variable if available or remove it
        if prev_http_proxy:
            os.environ["HTTP_PROXY"] = prev_http_proxy
        elif "HTTP_PROXY" in os.environ:
            del os.environ["HTTP_PROXY"]
        if prev_https_proxy:
            os.environ["HTTPS_PROXY"] = prev_https_proxy
        elif "HTTPS_PROXY" in os.environ:
            del os.environ["HTTPS_PROXY"]
        return result

    return wrapper


# #############################################################################
# ##### Stand alone program ########
# ##################################

if __name__ == "__main__":
    """Standalone execution."""
    pass



#! python3  # noqa: E265

"""Setup script to package into a Python module."""

# ############################################################################
# ########## Libraries #############
# ##################################

# standard library
from pathlib import Path

# 3rd party
from setuptools import find_packages, setup

# package (to get version)
from qgis_deployment_toolbelt import __about__

# ############################################################################
# ########### Globals ##############
# ##################################

# The directory containing this file
HERE = Path(__file__).parent

# The text of the README file
README = (HERE / "README.md").read_text()

# ############################################################################
# ########### Functions ############
# ##################################


def load_requirements(requirements_files: Path | list[Path]) -> list:
    """Helper to load requirements list from a path or a list of paths.

    Args:
        requirements_files (Path | list[Path]): path or list to paths of requirements
            file(s)

    Returns:
        list: list of requirements loaded from file(s)
    """
    out_requirements = []

    if isinstance(requirements_files, Path):
        requirements_files = [
            requirements_files,
        ]

    for requirements_file in requirements_files:
        with requirements_file.open(encoding="UTF-8") as f:
            out_requirements += [
                line
                for line in f.read().splitlines()
                if not line.startswith(("#", "-")) and len(line)
            ]

    return out_requirements


# ############################################################################
# ########### Main #################
# ##################################

setup(
    name=__about__.__package_name__,
    author=__about__.__author__,
    author_email=__about__.__email__,
    description=__about__.__summary__,
    long_description=README,
    long_description_content_type="text/markdown",
    keywords=__about__.__keywords__,
    url=__about__.__uri__,
    version=__about__.__version__,
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Information Technology",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: Microsoft :: Windows :: Windows 10",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
        "Operating System :: OS Independent",
        "Topic :: Scientific/Engineering :: GIS",
        "Topic :: System :: Installation/Setup",
    ],
    # packaging
    python_requires=">=3.10,<4",
    py_modules=["qgis_deployment_toolbelt"],
    packages=find_packages(
        exclude=["contrib", "docs", "*.tests", "*.tests.*", "tests.*", "tests", ".venv"]
    ),
    include_package_data=True,
    install_requires=load_requirements(HERE / "requirements/base.txt"),
    extras_require={
        # tooling
        "dev": load_requirements(HERE / "requirements/development.txt"),
        "doc": load_requirements(HERE / "requirements/documentation.txt"),
        "test": load_requirements(HERE / "requirements/testing.txt"),
    },
    entry_points={
        "console_scripts": [
            f"{__about__.__executable_name__}=qgis_deployment_toolbelt.cli:main",
            "qdeploy-toolbelt=qgis_deployment_toolbelt.cli:main",
            "qdt=qgis_deployment_toolbelt.cli:main",
        ]
    },
)



#! python3  # noqa: E265

"""Convert PNGF file(s) to multisizes ico.

Widely inspired from:

- png2ico (https://github.com/dbconfig/png2ico), MIT
- PyInstaller (https://github.com/pyinstaller/pyinstaller/blob/c7ff86f871d064110452562ed87c4fb95d2a718e/PyInstaller/building/icon.py), GPL2 or later
"""

# #############################################################################
# ########## Libraries #############
# ##################################

# standard library
import logging
from pathlib import Path

# 3rd party
try:
    from PIL import Image, UnidentifiedImageError
except ImportError as error:
    import sys

    sys.exit(
        "Please install Pillow ('pip install pillow') to make image conversion. "
        f"Trace: {error}"
    )

# package
from qgis_deployment_toolbelt.utils.check_path import check_path

# #############################################################################
# ########## Globals ###############
# ##################################

logger = logging.getLogger(__name__)

AUTO_RESIZE_OUTPUT_ICO_SIZES: tuple[tuple[int, int], ...] = (
    (16, 16),
    (24, 24),
    (32, 32),
    (48, 48),
    (64, 64),
    (128, 128),
    (255, 255),
)


# #############################################################################
# ########## Functions #############
# ##################################


def png2ico(in_png_path: Path, out_ico_path: Path | None = None) -> Path:
    """Convert a PNG file to an multisize ICO.

    Args:
        in_png_path (Path): input PNG image file path.
        out_ico_path (Path | None): output ico file path, optional.

    Raises:
        error: if input file does not exist or something goes wrong during conversion.

    Returns:
        Path: output ico file path
    """
    # check if input file exists
    check_path(
        input_path=in_png_path,
        must_be_a_file=True,
        must_be_a_folder=False,
        must_be_readable=True,
        must_exists=True,
    )

    # output path
    if not isinstance(out_ico_path, Path):
        out_ico_path = in_png_path.with_suffix(".ico")

    try:
        with Image.open(in_png_path) as im:
            # If an image uses a custom palette + transparency, convert it to RGBA for a
            # better alpha mask depth.
            if im.mode == "P" and im.info.get("transparency", None) is not None:
                # The bit depth of the alpha channel will be higher, and the images will
                # look better when eventually scaled to multiple sizes (16,24,32,..) for
                # the ICO format for example.
                im = im.convert("RGBA")
            im.save(out_ico_path, sizes=AUTO_RESIZE_OUTPUT_ICO_SIZES)
    except UnidentifiedImageError as error:
        logger.error(
            f"Something went wrong converting icon image '{in_png_path}' to .ico with "
            "Pillow,"
        )
        raise error

    return out_ico_path


# #############################################################################
# ##### Stand alone program ########
# ##################################

if __name__ == "__main__":
    """Standalone execution."""
    pass



#! python3  # noqa E265

"""
    Usage from the repo root folder:

    .. code-block:: bash
        # for whole tests
        python -m unittest tests.test_utils_proxies
        # for specific test
        python -m unittest tests.test_utils_proxies.TestUtilsNetworkProxies.test_proxy_settings
"""

# standard library
import unittest
from os import environ
from pathlib import Path

# project
from qgis_deployment_toolbelt.utils.proxies import get_proxy_settings

# ############################################################################
# ########## Classes #############
# ################################


class TestUtilsNetworkProxies(unittest.TestCase):
    """Test network proxy utilities."""

    def test_proxy_settings(self):
        """Test proxy settings retriever."""
        # by default, no proxy
        self.assertDictEqual(get_proxy_settings(), {})

        # using generic - only http
        get_proxy_settings.cache_clear()
        environ["HTTP_PROXY"] = "http://proxy.example.com:3128"
        self.assertIsInstance(get_proxy_settings(), dict)
        self.assertEqual(
            get_proxy_settings().get("http"), "http://proxy.example.com:3128"
        )
        self.assertIsNone(get_proxy_settings().get("https"))

        environ.pop("HTTP_PROXY")  # clean up

        # using generic - only https
        get_proxy_settings.cache_clear()
        environ["HTTPS_PROXY"] = "https://proxy.example.com:3128"
        self.assertIsInstance(get_proxy_settings(), dict)
        self.assertEqual(
            get_proxy_settings().get("https"), "https://proxy.example.com:3128"
        )
        self.assertIsNone(get_proxy_settings().get("http"))

        environ.pop("HTTPS_PROXY")  # clean up

        # using generic - both http and https
        get_proxy_settings.cache_clear()
        environ["HTTP_PROXY"] = "http://proxy.example.com:3128"
        environ["HTTPS_PROXY"] = "https://proxy.example.com:3128"
        self.assertIsInstance(get_proxy_settings(), dict)
        self.assertEqual(
            get_proxy_settings().get("https"), "https://proxy.example.com:3128"
        )
        self.assertEqual(
            get_proxy_settings().get("http"), "http://proxy.example.com:3128"
        )

        environ.pop("HTTP_PROXY")  # clean up
        environ.pop("HTTPS_PROXY")  # clean up

        # using custom QDT
        get_proxy_settings.cache_clear()
        environ["QDT_PROXY_HTTP"] = "http://user:p8ùX45@proxy.example.com:3128"
        self.assertIsInstance(get_proxy_settings(), dict)
        self.assertEqual(
            get_proxy_settings().get("http"),
            "http://user:p8ùX45@proxy.example.com:3128",
        )
        self.assertEqual(
            get_proxy_settings().get("https"),
            "http://user:p8ùX45@proxy.example.com:3128",
        )

        environ.pop("QDT_PROXY_HTTP")  # clean up

        # not valid URL - just to check the case
        environ["QDT_PROXY_HTTP"] = "socks5://user:motdepasse@proxy.example.com:1182"
        environ.pop("QDT_PROXY_HTTP")  # clean up
        get_proxy_settings.cache_clear()

    def test_pac_file(self):
        """Test PAC file proxies retriver"""

        get_proxy_settings.cache_clear()

        ## url
        environ["QDT_PAC_FILE"] = (
            "https://raw.githubusercontent.com/qgis-deployment/qgis-deployment-toolbelt-cli/refs/heads/main/tests/fixtures/pac/proxy.pac"
        )

        ### QGIS plugin : use proxy
        qgis_plugin_proxy_settings = get_proxy_settings(
            "https://plugins.qgis.org/plugins/french_locator_filter/version/1.1.1/download/"
        )
        self.assertIsInstance(qgis_plugin_proxy_settings, dict)
        self.assertEqual(
            qgis_plugin_proxy_settings.get("http"),
            "http://myproxy:8080",  # NOSONAR
        )
        self.assertEqual(
            qgis_plugin_proxy_settings.get("https"),
            "http://myproxy:8080",  # NOSONAR
        )

        ### In no proxy rules
        grand_plugin_proxy_settings = get_proxy_settings(
            "https://qgis-plugin.no-proxy.fr/plugin.zip"
        )
        self.assertIsInstance(grand_plugin_proxy_settings, dict)
        self.assertIsNone(grand_plugin_proxy_settings.get("http"))
        self.assertIsNone(grand_plugin_proxy_settings.get("https"))

        ### No url
        no_url_proxy_settings = get_proxy_settings()
        self.assertIsInstance(no_url_proxy_settings, dict)
        self.assertEqual(
            no_url_proxy_settings.get("http"),
            "http://myproxy:8080",  # NOSONAR
        )
        self.assertEqual(
            no_url_proxy_settings.get("http"),
            "http://myproxy:8080",  # NOSONAR
        )

        ## Local file
        get_proxy_settings.cache_clear()
        pac_file = Path("tests/fixtures/pac/proxy.pac")
        environ["QDT_PAC_FILE"] = str(pac_file.absolute())

        ### QGIS plugin : use proxy
        qgis_plugin_proxy_settings = get_proxy_settings(
            "https://plugins.qgis.org/plugins/french_locator_filter/version/1.1.1/download/"
        )
        self.assertIsInstance(qgis_plugin_proxy_settings, dict)
        self.assertEqual(
            qgis_plugin_proxy_settings.get("http"),
            "http://myproxy:8080",  # NOSONAR
        )
        self.assertEqual(
            qgis_plugin_proxy_settings.get("https"),
            "http://myproxy:8080",  # NOSONAR
        )

        ### In no proxy rules
        grand_plugin_proxy_settings = get_proxy_settings(
            "https://qgis-plugin.no-proxy.fr/plugin.zip"
        )
        self.assertIsInstance(grand_plugin_proxy_settings, dict)
        self.assertIsNone(grand_plugin_proxy_settings.get("http"))
        self.assertIsNone(grand_plugin_proxy_settings.get("https"))

        ### No url
        no_url_proxy_settings = get_proxy_settings()
        self.assertIsInstance(no_url_proxy_settings, dict)
        self.assertEqual(
            no_url_proxy_settings.get("http"),
            "http://myproxy:8080",  # NOSONAR
        )
        self.assertEqual(
            no_url_proxy_settings.get("http"),
            "http://myproxy:8080",  # NOSONAR
        )

        environ.pop("QDT_PAC_FILE")  # clean up
        get_proxy_settings.cache_clear()


# ############################################################################
# ####### Stand-alone run ########
# ################################
if __name__ == "__main__":
    unittest.main()



#! python3  # noqa: E265

"""
    Handle remote git repository.

    Author: Julien Moura (https://github.com/guts).

    Inspired from: QGIS Resource Sharing
"""


# #############################################################################
# ########## Libraries #############
# ##################################

# Standard library
import logging

# project
from qgis_deployment_toolbelt.profiles.profiles_handler_base import (
    RemoteProfilesHandlerBase,
)

# #############################################################################
# ########## Globals ###############
# ##################################

# logs
logger = logging.getLogger(__name__)


# #############################################################################
# ########## Classes ###############
# ##################################
class RemoteGitHandler(RemoteProfilesHandlerBase):
    """Handle remote git repository."""

    def __init__(
        self,
        source_repository_url: str,
        source_repository_type: str = "git_remote",
        branch_to_use: str | None = None,
    ) -> None:
        """Constructor.

        Args:
            source_repository_url (Union[str, Path]): input URI (http://, https://, git://)

        """
        super().__init__(
            source_repository_type=source_repository_type, branch_to_use=branch_to_use
        )

        self.SOURCE_REPOSITORY_PATH_OR_URL = source_repository_url

        # validation
        self.is_valid_git_repository()

        # check if passed branch exist
        if branch_to_use is None:
            self.DESTINATION_BRANCH_TO_USE = self.SOURCE_REPOSITORY_ACTIVE_BRANCH
            logger.info(
                "No branch specified. The default branch of source repository "
                f"({self.SOURCE_REPOSITORY_PATH_OR_URL}) will be used."
            )
        else:
            if not self.is_branch_existing_in_repository(branch_name=branch_to_use):
                logger.error(
                    f"Specified branch '{branch_to_use}' has not been found in source "
                    f"repository ({self.SOURCE_REPOSITORY_PATH_OR_URL}). Fallback to "
                    f"identified active branch: {self.SOURCE_REPOSITORY_ACTIVE_BRANCH}."
                )
                self.DESTINATION_BRANCH_TO_USE = self.SOURCE_REPOSITORY_ACTIVE_BRANCH
            else:
                self.DESTINATION_BRANCH_TO_USE = branch_to_use


# #############################################################################
# ##### Stand alone program ########
# ##################################

if __name__ == "__main__":
    """Standalone execution."""
    pass



#! python3  # noqa E265

"""Usage from the repo root folder:

    .. code-block:: python

        # for whole test
        python -m unittest tests.test_job_generic
        # for specific
        python -m unittest tests.test_job_generic.TestJobGeneric.test_listing_profiles_folder
"""

# #############################################################################
# ########## Libraries #############
# ##################################

# Standard library
import tempfile
import unittest
from pathlib import Path

# package
from qgis_deployment_toolbelt.exceptions import (
    JobOptionBadName,
    JobOptionBadValue,
    JobOptionBadValueType,
)
from qgis_deployment_toolbelt.jobs.generic_job import GenericJob

# #############################################################################
# ########## Classes ###############
# ##################################


class TestJobGeneric(unittest.TestCase):
    """Test generic job."""

    # -- Standard methods --------------------------------------------------------
    @classmethod
    def setUpClass(cls):
        """Executed when module is loaded before any test."""
        cls.generic_job = GenericJob()

        cls.generic_job.ID = "job-test-fixture"
        cls.generic_job.OPTIONS_SCHEMA = {
            "option_one": {
                "type": str,
                "required": True,
                "default": "git",
                "condition": "startswith",
                "possible_values": ("git", "http"),
            },
            "option_two": {
                "type": (bool, int, str),
                "required": False,
                "condition": "in",
                "possible_values": ("test", "fixture"),
            },
            "option_three": {
                "type": (bool, int, str),
                "required": False,
                "condition": None,
                "possible_values": None,
            },
        }

    # -- TESTS ---------------------------------------------------------
    def test_listing_profiles_folder(self):
        """Test profiles listing."""
        fixtures_profiles_folder = Path("tests/fixtures/profiles")
        with tempfile.TemporaryDirectory(
            prefix="QDT_test_profiles_listing_",
            ignore_cleanup_errors=True,
        ) as tmpdirname:
            tmp_folder_path = Path(tmpdirname).joinpath("profiles")
            # simulate a QGIS profiles folder structure in the temp folder
            for p in fixtures_profiles_folder.glob("good_profile_*.json"):
                dest_file = tmp_folder_path.joinpath(f"test_{p.stem}/profile.json")
                dest_file.parent.mkdir(parents=True, exist_ok=True)
                dest_file.write_text(p.read_text(encoding="UTF-8"), encoding="UTF-8")

            filtered_profiles = self.generic_job.filter_profiles_folder(
                start_parent_folder=tmp_folder_path
            )

            self.assertIsInstance(filtered_profiles, tuple)
            self.assertGreater(len(filtered_profiles), 0)

            # length must be -1 because of a profile which is excluded by a rule
            self.assertEqual(
                len(filtered_profiles),
                len(list(tmp_folder_path.glob("**/profile.json"))) - 1,
            )

    def test_validate_options_bad_input_type(self):
        """Test validate_options method"""

        # Options must be a dictionary
        with self.assertRaises(TypeError):
            self.generic_job.validate_options(options="options_test")
        with self.assertRaises(TypeError):
            self.generic_job.validate_options(options=["options_test"])

    def test_validate_options_bad_name(self):
        """Test validate_options method"""
        # bad option name
        bad_option_name = {
            "option_one": "http://fake.url",
            "option_two": "test",
            "bad_option_name": "git",
        }

        with self.assertRaises(JobOptionBadName):
            self.generic_job.validate_options(bad_option_name)

    def test_validate_options_bad_condition_in(self):
        """Test validate_options method"""
        # bad value condition
        bad_options_scope = {
            "option_one": "https://fake.url",
            "option_two": "bad-value-not-in-possibles",
        }

        with self.assertRaises(JobOptionBadValue):
            self.generic_job.validate_options(bad_options_scope)

    def test_validate_options_bad_condition_startswith(self):
        """Test validate_options method"""
        # bad value condition
        bad_options_scope = {
            "option_one": "bad-startswith",
            "option_two": "user",
        }

        with self.assertRaises(JobOptionBadValue):
            self.generic_job.validate_options(bad_options_scope)

    def test_validate_options_bad_type(self):
        """Test validate_options method"""
        # bad value type
        bad_options = {
            "option_one": True,
            "option_two": "user",
        }

        with self.assertRaises(JobOptionBadValueType):
            self.generic_job.validate_options(bad_options)
